@@ -64,6 +64,8 @@ typedef enum _HdWmClientType
 static void
 mb_wm_comp_mgr_clutter_add_actor (MBWMCompMgrClutter *,
 				  MBWMCompMgrClutterClient *);
+static void
+mb_wm_comp_mgr_clutter_fetch_texture (MBWMCompMgrClient *client);
 
 /**
  * Implementation of MBWMCompMgrClutterClient.
@@ -178,11 +180,7 @@ mb_wm_comp_mgr_clutter_set_client_redirection (MBWMCompMgrClient *client,
   MBWMCompMgrClutterClient *cclient = MB_WM_COMP_MGR_CLUTTER_CLIENT(client);
   Window xwin = None;
 
-  mb_wm_util_trap_x_errors ();
-  if (cclient->priv->texture)
-    clutter_x11_texture_pixmap_set_redirection (
-          CLUTTER_X11_TEXTURE_PIXMAP (cclient->priv->texture),
-          setting);
+  mb_wm_util_async_trap_x_errors (client->wm->xdpy);
 
   if (client->wm_client)
     {
@@ -201,7 +199,6 @@ mb_wm_comp_mgr_clutter_set_client_redirection (MBWMCompMgrClient *client,
           client->wm_client->xwin_frame == 0 ||
             (c_type & (MBWMClientTypeDialog|
                        MBWMClientTypeMenu|
-                       MBWMClientTypeNote|
                        MBWMClientTypeOverride)))
         xwin = client->wm_client->window->xwindow;
       else
@@ -211,23 +208,30 @@ mb_wm_comp_mgr_clutter_set_client_redirection (MBWMCompMgrClient *client,
       */
     }
 
+  cclient->priv->unredirected = setting ? False : True;
+
   if (setting && xwin != None)
     {
-      XCompositeRedirectSubwindows (client->wm->xdpy, xwin,
-                                    CompositeRedirectAutomatic);
       XCompositeRedirectWindow (client->wm->xdpy, xwin,
                                 CompositeRedirectManual);
+      XCompositeRedirectSubwindows (client->wm->xdpy, xwin,
+                                          CompositeRedirectAutomatic);
+      /* Fetching the texture here causes the texture to be updated
+       * with the correct window. */
+      mb_wm_comp_mgr_clutter_fetch_texture(MB_WM_COMP_MGR_CLIENT(cclient));
     }
   else if (xwin != None)
     {
-      XCompositeUnredirectWindow (client->wm->xdpy, xwin,
-                                  CompositeRedirectManual);
+      /* Fetching the texture here causes the texture's window to be
+       * set to 0, so it doesn't try to do anything with it. */
+      mb_wm_comp_mgr_clutter_fetch_texture(MB_WM_COMP_MGR_CLIENT(cclient));
       XCompositeUnredirectSubwindows (client->wm->xdpy, xwin,
                                       CompositeRedirectAutomatic);
+      XCompositeUnredirectWindow (client->wm->xdpy, xwin,
+                                        CompositeRedirectManual);
     }
-  cclient->priv->unredirected = setting ? False : True;
-  XSync (client->wm->xdpy, False);
-  mb_wm_util_untrap_x_errors ();
+
+  mb_wm_util_async_untrap_x_errors ();
 }
 
 /**
@@ -259,21 +263,19 @@ mb_wm_comp_mgr_clutter_fetch_texture (MBWMCompMgrClient *client)
    * if you just set the same window */
   clutter_x11_texture_pixmap_set_window (
           CLUTTER_X11_TEXTURE_PIXMAP (cclient->priv->texture),
-          0, FALSE);
+          0);
 
-  /* this will also cause updating the corresponding pixmap
-   * and ensures window<->pixmap binding */
-  mb_wm_util_trap_x_errors();
-  clutter_x11_texture_pixmap_set_window (
-        CLUTTER_X11_TEXTURE_PIXMAP (cclient->priv->texture),
-        xwin, FALSE);
-
-  if (mb_wm_util_untrap_x_errors () == BadDrawable)
+  /* If we are unredirected, just don't set windows, as we won't be able
+   * to get the pixmap for them anyway. */
+  if (!cclient->priv->unredirected)
     {
-      g_debug ("%s: BadDrawable for %lx", __FUNCTION__,
-               client->wm_client->window->xwindow);
-      cclient->priv->bound = FALSE;
-      return;
+      /* this will also cause updating the corresponding pixmap
+       * and ensures window<->pixmap binding. Clutter will handle
+       * any X error itself. */
+
+      clutter_x11_texture_pixmap_set_window (
+            CLUTTER_X11_TEXTURE_PIXMAP (cclient->priv->texture),
+            xwin);
     }
 
   cclient->priv->bound = TRUE;
@@ -723,7 +725,7 @@ mb_wm_comp_mgr_clutter_turn_on_real (MBWMCompMgr *mgr)
         return;
       }
 
-      mb_wm_util_trap_x_errors();
+      mb_wm_util_async_trap_x_errors_warn(wm->xdpy, "");
 
       /* Make sure the overlay window's size is the same as the screen's
        * actual size.  Necessary if the screen is rotated. */
@@ -738,7 +740,10 @@ mb_wm_comp_mgr_clutter_turn_on_real (MBWMCompMgr *mgr)
        * Reparent the stage window to the overlay window, this makes it
        * magically work :)
        */
+      XSetWindowBackgroundPixmap (wm->xdpy, xwin, None);
       XReparentWindow (wm->xdpy, xwin, priv->overlay_window, 0, 0);
+      XSetWindowBackground (wm->xdpy, xwin,
+                            BlackPixel (wm->xdpy, DefaultScreen (wm->xdpy)));
 
       /*
        * Use xfixes shape to make events pass through the overlay window
@@ -757,9 +762,7 @@ mb_wm_comp_mgr_clutter_turn_on_real (MBWMCompMgr *mgr)
 
       XFixesDestroyRegion (wm->xdpy, region);
 
-      XSync (wm->xdpy, False);
-      if (mb_wm_util_untrap_x_errors())
-        g_warning ("%s: X errors", __func__);
+      mb_wm_util_async_untrap_x_errors();
 
       clutter_actor_set_size (stage, wm->xdpy_width, wm->xdpy_height);
       clutter_stage_set_color (CLUTTER_STAGE (stage), &clr);
@@ -779,7 +782,8 @@ mb_wm_comp_mgr_clutter_screen_size_changed (MBWMCompMgr *mgr,
   MB_WM_DBG_MOVE_RESIZE ("overlay", priv->overlay_window,
                          (&(MBGeometry){0, 0, w, h}));
   clutter_actor_set_size (clutter_stage_get_default (), w, h);
-  XResizeWindow (mgr->wm->xdpy, priv->overlay_window, w, h);
+  if (priv->overlay_window != None)
+    XResizeWindow (mgr->wm->xdpy, priv->overlay_window, w, h);
 }
 
 static void
@@ -885,7 +889,6 @@ mb_wm_comp_mgr_clutter_handle_damage (XDamageNotifyEvent * de,
   if (!cclient->priv->damage_handling_off)
     {
       Damage damage;
-      int err;
 
 /* We ignore the DontUpdate flag for i386, as it uses the X11 Texture Pixmap
  * class, which requires damage events to keep its internal texture in sync.
@@ -918,10 +921,9 @@ mb_wm_comp_mgr_clutter_handle_damage (XDamageNotifyEvent * de,
 
       /* FIXME: As Adam said, reason for this X error should be discovered
        * and avoided */
-      mb_wm_util_trap_x_errors ();
+      mb_wm_util_async_trap_x_errors_warn (wm->xdpy, "");
       mb_wm_comp_mgr_clutter_client_repair_real (c->cm_client, damage);
-      if ((err = mb_wm_util_untrap_x_errors ()))
-        g_debug ("%s: X error %d", __func__, err);
+      mb_wm_util_async_untrap_x_errors();
     }
 
   return False;
@@ -1104,13 +1106,11 @@ mb_wm_comp_mgr_clutter_client_track_damage (MBWMCompMgrClutterClient *cclient,
     {
       if (cclient->priv->window_damage)
         {
-          int err;
-          mb_wm_util_trap_x_errors();
+          /* When the window is closing, this may fail - which we don't
+           * really care about, so don't report it */
+          mb_wm_util_async_trap_x_errors(wm->xdpy);
           XDamageDestroy (wm->xdpy, cclient->priv->window_damage);
-          XSync (wm->xdpy, False);
-          if ((err = mb_wm_util_untrap_x_errors()) != 0)
-            g_debug ("XDamageDestroy(0x%lx) for %p: %d",
-                     cclient->priv->window_damage, c, err);
+          mb_wm_util_async_untrap_x_errors();
           cclient->priv->window_damage = 0;
         }
 
@@ -1118,7 +1118,7 @@ mb_wm_comp_mgr_clutter_client_track_damage (MBWMCompMgrClutterClient *cclient,
         /* release the window in Clutter */
         clutter_x11_texture_pixmap_set_window (
                 CLUTTER_X11_TEXTURE_PIXMAP (cclient->priv->texture),
-                0, FALSE);
+                0);
       cclient->priv->damage_handling_off = True;
     }
 }
@@ -1310,3 +1310,10 @@ mb_wm_comp_mgr_clutter_get_overlay_window (MBWMCompMgrClutter *cmgr)
 {
   return cmgr->priv->overlay_window;
 }
+
+void
+mb_wm_comp_mgr_clutter_set_overlay_window (MBWMCompMgrClutter *cmgr, Window w)
+{
+  cmgr->priv->overlay_window = w;
+}
+
