@@ -45,6 +45,7 @@ enum {
   COOKIE_WIN_HILDON_STACKING,
   COOKIE_WIN_HILDON_TYPE,
   COOKIE_WIN_PORTRAIT_REQUEST,
+  COOKIE_WIN_NO_TRANSITIONS,
 
   N_COOKIES
 };
@@ -294,6 +295,13 @@ mb_wm_client_window_sync_properties ( MBWMClientWindow *win,
 				       wm->atoms[MBWM_ATOM_NET_WM_PID]);
     }
 
+  if (props_req & MBWM_WINDOW_PROP_NO_TRANSITIONS)
+    {
+      cookies[COOKIE_WIN_NO_TRANSITIONS]
+	= mb_wm_property_cardinal_req (wm, xwin,
+	        wm->atoms[MBWM_ATOM_HILDON_WM_ACTION_NO_TRANSITIONS]);
+    }
+
   if (props_req & MBWM_WINDOW_PROP_NET_USER_TIME)
     {
       cookies[COOKIE_WIN_USER_TIME]
@@ -325,13 +333,12 @@ mb_wm_client_window_sync_properties ( MBWMClientWindow *win,
     }
 
   {
-    int err;
     /* FIXME: toggling 'offline' mode in power menu can cause X error here */
-    mb_wm_util_trap_x_errors ();
-    /* bundle all pending requests to server and wait for replys */
+    /* bundle all pending requests to server and wait for replys.
+     * Errors will be caught by the new error handler. Note that removing this
+     * absolutely kills hildon-desktop. It appears that the property specifying
+     * window type doesn't get read and everything gets mapped as an app.  */
     XSync(wm->xdpy, False);
-    if ((err = mb_wm_util_untrap_x_errors ()))
-      g_warning ("%s: X error %d", __func__, err);
   }
 
   if (props_req & MBWM_WINDOW_PROP_TRANSIENCY)
@@ -429,7 +436,7 @@ mb_wm_client_window_sync_properties ( MBWMClientWindow *win,
 	  || result_atom == NULL
 	  )
 	{
-	  g_debug ("%s: ### Warning net type prop failed for %x ###",
+	  g_debug ("%s: ### Warning net type prop failed for %lx ###",
 		   __FUNCTION__, xwin);
 	  if (x_error_code == BadWindow)
 	    goto badwindow_error;
@@ -455,7 +462,14 @@ mb_wm_client_window_sync_properties ( MBWMClientWindow *win,
 	  if (win->net_type != ((Atom *)result_atom)[0])
 	    changes |= MBWM_WINDOW_PROP_WIN_TYPE;
 
-	  win->net_type = ((Atom*)result_atom)[0];
+          if (((Atom*)result_atom)[0]
+                          == wm->atoms[MBWM_ATOM_NET_WM_WINDOW_TYPE_DND])
+            {
+              g_warning ("%s: treat DND window as normal\n", __func__);
+              win->net_type = wm->atoms[MBWM_ATOM_NET_WM_WINDOW_TYPE_NORMAL];
+            }
+          else
+	    win->net_type = ((Atom*)result_atom)[0];
 	}
 
       if (result_atom)
@@ -1030,6 +1044,42 @@ mb_wm_client_window_sync_properties ( MBWMClientWindow *win,
 	XFree (translucency);
     }
 
+  if (props_req & MBWM_WINDOW_PROP_NO_TRANSITIONS)
+    {
+      mb_wm_property_reply (wm,
+			    cookies[COOKIE_WIN_NO_TRANSITIONS],
+			    &actual_type_return,
+			    &actual_format_return,
+			    &nitems_return,
+			    &bytes_after_return,
+			    &result_atom,
+			    &x_error_code);
+
+      cookies[COOKIE_WIN_NO_TRANSITIONS] = 0;
+
+      if (x_error_code
+	  || actual_type_return != XA_CARDINAL
+	  || actual_format_return != 32
+	  || result_atom == NULL
+	  )
+	{
+	  MBWM_DBG("### Warning no transitions prop failed ###");
+          if (x_error_code == BadWindow)
+            goto badwindow_error;
+	}
+      else
+	{
+	  win->allowed_actions |= MBWMClientWindowActionNoTransitions;
+	}
+
+      if (result_atom)
+	XFree(result_atom);
+
+      changes |= MBWM_WINDOW_PROP_NO_TRANSITIONS;
+
+      result_atom = NULL;
+    }
+
   if (props_req & MBWM_WINDOW_PROP_NET_USER_TIME)
     {
       unsigned char *user_time = NULL;
@@ -1269,6 +1319,7 @@ badwindow_error:
                         COOKIE_WIN_NET_STATE,
                         COOKIE_WIN_TYPE,
                         COOKIE_WIN_HILDON_TYPE,
+                        COOKIE_WIN_NO_TRANSITIONS,
                         0
     };
     int *cursor;
