@@ -38,6 +38,7 @@ struct MBWindowManagerClientPriv
   Bool          realized;
   Bool          mapped;
   Bool          map_confirmed;
+  Bool          unmap_confirmed;
   Bool          iconizing;
   Bool          hiding_from_desktop;
   MBWMSyncType  sync_state;
@@ -485,11 +486,23 @@ mb_wm_client_is_map_confirmed (MBWindowManagerClient *client)
 
 void
 mb_wm_client_set_map_confirmed (MBWindowManagerClient *client,
-		Bool confirmed)
+                                Bool confirmed)
 {
   client->priv->map_confirmed = confirmed;
 }
 
+Bool
+mb_wm_client_is_unmap_confirmed (MBWindowManagerClient *client)
+{
+  return client->priv->unmap_confirmed;
+}
+
+void
+mb_wm_client_set_unmap_confirmed (MBWindowManagerClient *client,
+                                  Bool confirmed)
+{
+  client->priv->unmap_confirmed = confirmed;
+}
 
 void
 mb_wm_client_display_sync (MBWindowManagerClient *client)
@@ -656,9 +669,12 @@ MBWindowManagerClient*
 mb_wm_client_get_last_focused_transient (MBWindowManagerClient *client)
 {
   MBWindowManagerClient *result = NULL, *c;
+  MBWindowManager       *wm = client->wmref;
 
-  mb_wm_stack_enumerate_reverse (client->wmref, c)
-    if (c->transient_for == client)
+  mb_wm_stack_enumerate_reverse (wm, c)
+    if (c->transient_for == client &&
+        c->window->net_type != 
+	  wm->atoms[MBWM_ATOM_HILDON_WM_WINDOW_TYPE_ANIMATION_ACTOR])
       {
         result = c;
         break;
@@ -773,6 +789,7 @@ mb_wm_client_deliver_message (MBWindowManagerClient   *client,
   MBWindowManager *wm = client->wmref;
   Window xwin = client->window->xwindow;
   XEvent ev;
+  int status;
 
   memset(&ev, 0, sizeof(ev));
 
@@ -786,10 +803,12 @@ mb_wm_client_deliver_message (MBWindowManagerClient   *client,
   ev.xclient.data.l[3] = data3;
   ev.xclient.data.l[4] = data4;
 
-  mb_wm_util_trap_x_errors();
-  XSendEvent(wm->xdpy, xwin, False, NoEventMask, &ev);
-  XSync(wm->xdpy, False);
-  return mb_wm_util_untrap_x_errors()==0;
+  /* We can do this asynchronously because whenever this function is
+   * called when a sync is required, XSync is called after it. */
+  mb_wm_util_async_trap_x_errors_warn(wm->xdpy, "XSendEvent");
+  status = XSendEvent(wm->xdpy, xwin, False, NoEventMask, &ev);
+  mb_wm_util_async_untrap_x_errors();
+  return status!=0;
 }
 
 void
@@ -860,12 +879,16 @@ void
 mb_wm_client_shutdown (MBWindowManagerClient *client)
 {
   char               buf[257];
-  int                sig     = 9;
-  MBWindowManager   *wm      = client->wmref;
-  MBWMClientWindow  *win     = client->window;
-  Window             xwin    = client->window->xwindow;
-  const char        *machine = win->machine;
-  pid_t              pid     = win->pid;
+  Window             xwin;
+  const char        *machine;
+  pid_t              pid;
+
+  if (!client || !client->window)
+    return;
+
+  xwin = client->window->xwindow;
+  machine = client->window->machine;
+  pid = client->window->pid;
 
   if (machine && pid && (gethostname (buf, sizeof(buf)-1) == 0))
     {
@@ -878,14 +901,14 @@ mb_wm_client_shutdown (MBWindowManagerClient *client)
 	      return;
 	    }
 
-          g_debug ("%s: kill(%u)", __FUNCTION__, pid);
-	  if (kill (pid, sig) >= 0)
+          g_debug ("%s: kill(%u, SIGKILL)", __FUNCTION__, pid);
+	  if (kill (pid, SIGKILL) >= 0)
 	    return;
 	}
     }
 
   g_debug ("%s: XKillClient for %lx", __FUNCTION__, xwin);
-  XKillClient(wm->xdpy, xwin);
+  XKillClient(client->wmref->xdpy, xwin);
 }
 
 void
